import os
from multiprocessing import Pool
import sys

import numpy
import time

from operondemmo.version import kallisto_out_file


def check_kallisto():
    is_exist = os.system("kallisto version")
    if is_exist != 0:
        print("need kallisto. link: https://pachterlab.github.io/kallisto/download")
        sys.exit(1)


def split_from_input(input_files):
    fastq_files = []
    all_file = []
    for each_dir in os.listdir(input_files):
        all_file.append(input_files + each_dir + "/")
    for each_dir in all_file:
        condition_files = []
        condition_name = []
        for each_file in os.listdir(each_dir):
            condition_files.append(each_dir + each_file)
        for each_file in condition_files:
            tmp_each = each_file.split("/")[-1]
            tmp_each2 = tmp_each.split("_")[0]
            if tmp_each2 not in condition_name:
                condition_name.append(tmp_each2)
        if ".gz" in condition_files[0]:
            i = 0
            while i < len(condition_name):
                condition_name[i] = each_dir + condition_name[i] + "_1.fastq.gz " \
                                    + each_dir + condition_name[i] + "_2.fastq.gz"
                i = i + 1
        else:
            i = 0
            while i < len(condition_name):
                condition_name[i] = each_dir + condition_name[i] + "_1.fastq " \
                                    + each_dir + condition_name[i] + "_2.fastq"
                i = i + 1
        fastq_files.append(" ".join(condition_name))
    return fastq_files


def split_from_input_old(input_files):
    fna_file = ""
    all_files = []
    for each_file in os.listdir(input_files):
        all_files.append(input_files + each_file)
    for each_ in all_files:
        if ".fna" in each_:
            fna_file = each_
            all_files.remove(each_)
    if fna_file == "":
        print("need .fna file.")
        sys.exit(1)
    fastq_files = []
    for each_ in all_files:
        tmp_each = each_.split("/")[-1]
        tmp_each2 = tmp_each.split("_")[0]
        if tmp_each2 not in fastq_files:
            fastq_files.append(tmp_each2)
    if ".gz" in all_files[0]:
        i = 0
        while i < len(fastq_files):
            fastq_files[i] = input_files + fastq_files[i] + "_1.fastq.gz " \
                             + input_files + fastq_files[i] + "_2.fastq.gz"
            i = i + 1
    else:
        i = 0
        while i < len(fastq_files):
            fastq_files[i] = input_files + fastq_files[i] + "_1.fastq " + input_files + fastq_files[i] + "_2.fastq"
            i = i + 1
    # print(fastq_files)
    # print(fna_file, type(fna_file))
    return fna_file, fastq_files


def read_fna(fna_file):
    fna_fp = open(fna_file, 'r')
    fna_str = ""
    while True:
        line = fna_fp.readline().strip()
        if not line:
            break
        if ">" in line:
            pass
        else:
            fna_str = fna_str + line
    fna_fp.close()
    return fna_str


def to_fasta(gene_name, gene_str):
    gene_name = ">" + gene_name
    line_len = 80
    new_gene_str = gene_name
    i = 0
    while i < len(gene_str):
        new_gene_str = new_gene_str + "\n" + gene_str[i:i + line_len]
        i = i + line_len
    return new_gene_str


def frg_fna_according_to_gene_pos(fna_file, fna_path, gene_pos_dict, gene_sort):
    fna_str = read_fna(fna_file)
    fna_frg_fp = open(fna_path, 'w')
    fna_frg_list = []
    for gene in gene_sort:
        fna_frg_list.append("")
        for (start, stop) in gene_pos_dict[gene]:
            fna_frg_list[-1] = fna_frg_list[-1] + fna_str[start - 1:stop]
        fna_frg_fp.write(to_fasta(gene, fna_frg_list[-1]) + "\n")
    # print(len(fna_frg_list))
    fna_frg_fp.close()


def generate_kallisto_index(fna_file, gene_pos_dict, output_path, gene_sort):
    tmp_path = output_path + "tmp/"
    fna_name = fna_file.split("/")[-1]
    fna_path = tmp_path + "frg_" + fna_name
    if not os.path.exists(fna_path):
        frg_fna_according_to_gene_pos(fna_file, fna_path, gene_pos_dict, gene_sort)
    kallisto_index = fna_path.split(".")[0] + ".idx"
    if not os.path.exists(kallisto_index):
        os.system("kallisto index -i " + kallisto_index + " " + fna_path)
    return kallisto_index


def run_kallisto_quant_paired(fastq_files, kallisto_index, output_path):
    if not os.path.exists(output_path):
        is_ok = os.system("kallisto quant -i " + kallisto_index + " -o " + output_path + " " + fastq_files)
        if is_ok != 0:
            print("check your input. something wrong.")
            sys.exit(1)
    return output_path + kallisto_out_file


def get_tpm_from_kallisto_quant(kallisto_index, fastq_files, output_path, p):
    tmp_path = output_path + "tmp/"
    kallisto_out = []
    for i in range(len(fastq_files)):
        kallisto_out.append(tmp_path + str(i) + "_out/")
    kallisto_index_list = [kallisto_index] * len(fastq_files)
    start = time.time()
    pool = Pool(p)
    tpm_files = pool.starmap(run_kallisto_quant_paired, zip(fastq_files, kallisto_index_list, kallisto_out))
    pool.close()
    pool.join()
    end = time.time()
    print("time: compute tpm: %.2f" % (end - start))
    return tpm_files


def load_from_tpm_files(tpm_files):
    tmp_matrix = []
    for each_ in tpm_files:
        tmp_matrix.append([])
        file_fp = open(each_, 'r')
        file_fp.readline()
        file_content = file_fp.read().strip()
        content_list = file_content.split("\n")
        for line in content_list:
            tmp_content = line.split("\t")
            tmp_matrix[-1].append(tmp_content[-1])
    tmp_matrix = numpy.array(tmp_matrix).astype('float64').T
    # numpy.savetxt("/home/lyd/document/2018.1/gamma_domain/kallisto_matrix.txt", tmp_matrix, fmt="%.8f")
    # print(tmp_matrix.shape[0], tmp_matrix.shape[1])
    return tmp_matrix


if __name__ == "__main__":
    # test read_fna()
    fna_file_path = "/home/lyd/document/2018.1/eco/eco.fna"
    eco_fna = read_fna(fna_file_path)
    print(len(eco_fna))



import numpy


def get_person_by_condition(matrix_a):
    condition_num = matrix_a.shape[1]
    print("condition_num:", condition_num)
    gene_num = matrix_a.shape[0]
    print("gene_num:", gene_num)
    matrix_a_person = numpy.corrcoef(matrix_a, rowvar=True)
    return matrix_a_person


def compute_co_expression_by_person(matrix_a):
    matrix_condition_s_v = get_person_by_condition(matrix_a)
    return matrix_condition_s_v


if __name__ == "__main__":
    # test matrix_co_expression()
    c1 = [50, 33, 20, 19, 13]
    c2 = [51, 34, 21, 18, 12]
    c3 = [52, 35, 22, 17, 12]
    matrix_test = numpy.array([c1, c2, c3]).T
    print(compute_co_expression_by_person(matrix_test))



def get_operon_prior(sort_gene, gene_strand):
    number_direction = 0
    number_pair = 0
    pre_strand = "?"
    current_direction_len = 1
    i = 0
    while i < len(sort_gene):
        current_strand = gene_strand[sort_gene[i]]
        if current_strand != pre_strand and current_direction_len > 1:
            number_direction = number_direction + 1
            pre_strand = current_strand
            current_direction_len = 1
        elif current_strand != pre_strand and current_direction_len == 1:
            pre_strand = current_strand
            current_direction_len = 1
        elif current_strand == pre_strand and current_direction_len > 1:
            number_pair = number_pair + 1
            pre_strand = current_strand
            current_direction_len = current_direction_len + 1
        elif current_strand == pre_strand and current_direction_len == 1:
            number_pair = number_pair + 1
            pre_strand = current_strand
            current_direction_len = current_direction_len + 1
        else:
            pass
    if current_direction_len > 1:
        number_direction = number_direction + 1
    operon_prior = 1 - number_direction / number_pair
    non_operon_prior = 1 - operon_prior
    return operon_prior, non_operon_prior


def kernel_function(bandwidth, bin_size, samples, x_min, x_max):
    estimation_y = []
    estimation_x = []
    i = x_min
    while i <= x_max:
        _sum = 0
        for j in samples:
            if abs(i - j) <= bandwidth:
                tmp = (i - j) / bandwidth
                _sum = _sum + tmp * tmp
        value = _sum / (len(samples) * bandwidth)
        estimation_x.append(i)
        estimation_y.append(value)
        i = i + bin_size
    return estimation_x, estimation_y


def get_distance_distribution(gene_pos, gene_strand):
    gene_pos_list = []
    for key in gene_pos:
        for (start, stop) in gene_pos[key]:
            gene_pos_list.append((key, (start, stop)))
    gene_pos_list = sorted(gene_pos_list, key=lambda x: x[1][0])
    i = 1
    same_direction = []
    opposite_direction = []
    while i < len(gene_pos_list):
        length = gene_pos_list[i][1][0] - gene_pos_list[i - 1][1][1]
        if gene_strand[gene_pos_list[i][0]] == gene_strand[gene_pos_list[i - 1][0]]:
            same_direction.append(length)
        else:
            opposite_direction.append(length)
    distance_x, distance_y = kernel_function(30, 1, same_direction, -50, 200)
    return distance_x, distance_y


def get_co_expression_distribution(matrix_a):
    co_expression_list = matrix_a.diagonal(-1).tolist()
    co_x, co_y = kernel_function(5, 1, co_expression_list, -20, 20)
    return co_x, co_y






from multiprocessing import Pool

import numpy
import pandas


def get_spearman_by_condition_old(matrix_a):
    condition_num = matrix_a.shape[1]
    print("condition_num:", condition_num)
    gene_num = matrix_a.shape[0]
    print("gene_num:", gene_num)
    matrix_a_t = matrix_a.T
    matrix_a_t = pandas.DataFrame(matrix_a_t)
    matrix_a_spearman = numpy.array(matrix_a_t.corr('spearman'))
    return matrix_a_spearman


def compute_co_expression_by_spearman_old(matrix_a):
    matrix_condition_s_v = get_spearman_by_condition_old(matrix_a)
    return matrix_condition_s_v


def spearman_task(pd_matrix):
    pd_spearman = pd_matrix.corr('spearman')
    return pd_spearman[0][1]


def get_spearman_by_condition(matrix_a, p):
    condition_num = matrix_a.shape[1]
    print("condition_num:", condition_num)
    gene_num = matrix_a.shape[0]
    print("gene_num:", gene_num)
    list_g_i_g_i_plus_1 = []
    i = 1
    while i < gene_num:
        np_matrix = matrix_a[i - 1:i + 1].T
        pd_matrix = pandas.DataFrame(np_matrix)
        list_g_i_g_i_plus_1.append(pd_matrix)
        i = i + 1
    pool = Pool(p)
    list_a = pool.map(spearman_task, list_g_i_g_i_plus_1)
    matrix_spearman = numpy.diag(list_a, 1)
    return matrix_spearman


def compute_co_expression_by_spearman(matrix_a, p):
    matrix_condition_s_v = get_spearman_by_condition(matrix_a, p)
    return matrix_condition_s_v


if __name__ == "__main__":
    # test matrix_co_expression()
    c1 = [50, 33, 20, 19, 13]
    c2 = [51, 34, 21, 18, 12]
    c3 = [52, 35, 22, 17, 12]
    matrix_test = numpy.array([c1, c2, c3]).T
    print(compute_co_expression_by_spearman(matrix_test, 1))
    print(compute_co_expression_by_spearman_old(matrix_test))
